// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qpainter.h>

#include "clicklineedit.h"

KLDrawClickLineEdit::KLDrawClickLineEdit(const QString &msg, QWidget *parent, const char *name)
	: KLineEdit(parent, name)
{
	drawClickMsg_ = true;
	setClickMessage(msg);
}

void KLDrawClickLineEdit::setClickMessage(const QString &msg)
{
	clickMessage_ = msg;
	repaint();
}

void KLDrawClickLineEdit::setText(const QString &text)
{
	drawClickMsg_ = text.isEmpty();
	repaint();
	KLineEdit::setText(text);
}

void KLDrawClickLineEdit::drawContents(QPainter *p)
{
	KLineEdit::drawContents(p);
	
	if (drawClickMsg_ && !hasFocus()) {
		QPen tmp = p->pen();
		p->setPen(palette().color(QPalette::Disabled, QColorGroup::Text));
		QRect cr = contentsRect();
		
		cr.rLeft() += 3;
		p->drawText(cr, AlignAuto|AlignVCenter, clickMessage_);
		p->setPen(tmp);
	}
}

void KLDrawClickLineEdit::dropEvent(QDropEvent *ev)
{
	drawClickMsg_ = false;
	KLineEdit::dropEvent(ev);
}

void KLDrawClickLineEdit::focusInEvent(QFocusEvent *ev)
{
	if (drawClickMsg_) {
		drawClickMsg_ = false;
		repaint();
	}
	QLineEdit::focusInEvent(ev);
}

void KLDrawClickLineEdit::focusOutEvent(QFocusEvent *ev)
{
	if (text().isEmpty()) {
		drawClickMsg_ = true;
		repaint();
	}
	QLineEdit::focusOutEvent(ev);
}
