// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qbrush.h>
#include <qcolor.h>
#include <qpainter.h>
#include <kapplication.h>
#include <klocale.h>
#include <ktoolbar.h>

#include "colorpicker.h"

using namespace ldraw;

/////////////////////////////////////////////////////////////////////////////////////////
// KLDrawColorPicker
/////////////////////////////////////////////////////////////////////////////////////////

KLDrawColorPicker::KLDrawColorPicker(QWidget *parent, const char *name) : QFrame(parent, name)
{
	setFrameShape(QFrame::StyledPanel);
	setFrameShadow(QFrame::Sunken);
	setLineWidth(1);
	setSizeIncrement(20, 1);
	
	for (int i = 0; i < 16; ++i)
		addColor(color(i).get_entity());
}

void KLDrawColorPicker::addColor(const color_entity *c, int pos)
{
	if (pos == -1)
		colorList_.append(c);
	else
		colorList_.insert(pos, c);
}

void KLDrawColorPicker::paintEvent(QPaintEvent *e)
{
	QFrame::paintEvent(e);
	
	int margin = frameWidth() * 2;
	int x;
	QPainter p(this);
	
	p.setPen(Qt::NoPen);
	
	x = 0;
	QBrush b;
	
	for (const ldraw::color_entity *c = colorList_.first(); c; c = colorList_.next()) {
		b.setColor(QColor(c->rgba[0], c->rgba[1], c->rgba[2]));
		if (c->material == material_clear)
			b.setStyle(QBrush::Dense4Pattern);
		else
			b.setStyle(QBrush::SolidPattern);
		p.setBrush(b);
		p.drawRect(x+margin, margin, 20, height() - (margin * 2));
		x += 20;
	}
}

void KLDrawColorPicker::mouseReleaseEvent(QMouseEvent *e)
{
	int count = colorList_.count();
	int margin = frameWidth() * 2;
	
	if ((e->pos().x() - margin) >= count*20) return;
	
	emit colorPicked(colorList_.at((e->pos().x() - margin) / 20));
}

/////////////////////////////////////////////////////////////////////////////////////////
// KLDrawColorPickerAction
/////////////////////////////////////////////////////////////////////////////////////////

KLDrawColorPickerAction::KLDrawColorPickerAction(KActionCollection *ac)
	: KAction(i18n("Color Picker"), 0, ac, "color_picker")
{
	setShortcutConfigurable(false);
}

int KLDrawColorPickerAction::plug(QWidget *w, int index)
{
	KToolBar *bar = dynamic_cast<KToolBar *>(w);
	
	if (bar && kapp->authorizeKAction(name())) {
		const int id = KAction::getToolButtonID();
		
		addContainer(w, id);
		connect(w, SIGNAL(destroyed()), SLOT(slotDestroyed()));
		widget_ = new KLDrawColorPicker(w);
		bar->insertWidget(id, 0, widget_, index);
		bar->setItemAutoSized(id, true);
		
		return containerCount() - 1;
	} else
		return -1;
}
