// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _DOCUMENT_H_
#define _DOCUMENT_H_

#include <string>
#include <map>
#include <utility>

#include <qobject.h>
#include <qpixmap.h>
#include <qtabbar.h>
#include <kurl.h>

#include "libldr.h"

struct KLDrawViewport
{
	float left;
	float top;
	float right;
	float bottom;
	float aspectRatio;
};

extern const ldraw::matrix isometricProjectionMatrix;
extern const KLDrawViewport defaultViewport;

class QTimer;
class KLDrawDocument;
class KLDrawPartRenderer;

class KLDrawDocument : public QObject, public QTab
{
	Q_OBJECT;
	
	public:
		KLDrawDocument(int number, QObject *parent = 0L);
		KLDrawDocument(QString fileName, QObject *parent = 0L);
		~KLDrawDocument();
		
		ldraw::model_multipart* getModel() const { return modelBase_; }
		ldraw::model* getActiveModel() const { return activeModel_; }
		void setActiveModel(ldraw::model *m) { activeModel_ = m; }
		
		QPixmap* getPixmap() { return &modelPixmap_; }
		QPixmap* getSubmodelPixmap(std::string key);
		
		KURL& location() { return location_; }
		bool readOnly() { return readOnly_; }
		bool canSave() { return canSave_; }
		bool firstSave() { return firstSave_; }
		void setLocation(const KURL &u) { location_ = u; }
		void setReadOnly(bool s) { readOnly_ = s; }
		void setSaveable(bool s);
		void setFirstSave(bool s) { firstSave_ = s; }
		
		// Viewport-related stuff
		KLDrawViewport& getViewport(int i) { return viewport_[i]; }
		ldraw::matrix& getFreeViewport() { return freeViewport_; }
		ldraw::matrix& getLastRotation() { return lastRot_; }
		const ldraw::vector3d& getCenter() const { return center_; }
		const ldraw::vector3d& getLength() const { return length_; }
		
	public slots:
		// auto fit
		void recalibrateScreenDimension();
		void resetPerspective();
		
	private slots:
		void updatePixmap();
		
	private:
		QPixmap prettyPreview(ldraw::model *m);
		
		ldraw::model_multipart *modelBase_;
		ldraw::model *activeModel_;
		
		KLDrawViewport viewport_[7];
		ldraw::matrix freeViewport_;
		ldraw::matrix lastRot_;
		ldraw::vector3d center_;
		ldraw::vector3d length_;
		
		KURL location_;
		
		bool canSave_;
		bool readOnly_;
		bool firstSave_;
		
		KLDrawPartRenderer *renderer_;
		QPixmap modelPixmap_;
		std::map<std::string, QPixmap> submodelPixmap_;
		
		friend class KLDrawPixmapUpdater;
};

#endif
