// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qobject.h>

#include "libldr.h"

class KLDrawListItem;

class KLDrawEditor : public QObject
{
	Q_OBJECT;
	
	public:
		enum GridType { Sparse, Standard, Dense, Fine };
		
		KLDrawEditor(QObject *parent = 0L, const char *name = 0L);
		~KLDrawEditor();
		
		void setGridType(GridType t) { grid_ = t; }
		GridType getGridType() { return grid_; }
		float gridScale(int axis) const;
		
		void insertObject(const QString &filename, const ldraw::matrix &tr, const ldraw::color &col);
		void moveCurrentObject(const ldraw::matrix &m);
		void modifyMatrix(KLDrawListItem *item, const ldraw::matrix &m);
		
	signals:
		void modelModified();
		void needUpdate();
		void cursorChanged(KLDrawListItem *);
		
	public slots:
		void gridSparse();
		void gridStandard();
		void gridDense();
		void gridFine();
		
		void rotateXCW();
		void rotateXCCW();
		void rotateYCW();
		void rotateYCCW();
		void rotateZCW();
		void rotateZCCW();
		
		void moveXM();
		void moveXP();
		void moveYM();
		void moveYP();
		void moveZM();
		void moveZP();
		
		void changeColor(const ldraw::color_entity *);
		
		void remove();
		
	private:
		void changeModel();
		
		void doRotate(const ldraw::matrix &m);
		void moveRel(int axis, int dir);
		
		GridType grid_;
};
