// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _LIST_H_
#define _LIST_H_

#include <klistview.h>

#include "libldr.h"

class KLDrawDocument;
class KLDrawListItem;
class KLDrawTransformDialog;

class KLDrawList : public KListView
{
	Q_OBJECT;
	
	static KLDrawList *instance_;
	
	public:
		KLDrawList(QWidget *parent = 0L, const char *name = 0L);
		~KLDrawList();
		
		void clearSelection();
		
		static KLDrawList* instance() { return instance_; }
		
		KLDrawListItem* selected() { return selectedItem_; }
		ldraw::element_extref* lastRef() const;
		
	signals:
		void enableAction(int, bool);
		void cursorChanged(KLDrawListItem *);
		
	public slots:
		void updateList();
		void selectionChanged();
		
		void goFirst();
		void goPrev();
		void goNext();
		void goLast();
		
	private slots:
		void contextMenu(QListViewItem *item, const QPoint &pos, int col);
		
	private:
		KLDrawDocument **activeDocument_;
		KLDrawListItem *selectedItem_;
		
		KLDrawTransformDialog *modifyDialog_;
		
		int colNo_;
		int colItem_;
		
	protected:
		// Omit any keypress event
		void keyPressEvent(QKeyEvent *) {}
		void keyReleaseEvent(QKeyEvent *) {}
};

class KLDrawListItem : public KListViewItem
{
	public:
		KLDrawListItem(KLDrawList *parent, int number, ldraw::element_base *item);
		KLDrawListItem(KLDrawList *parent, KLDrawListItem *after, int number, ldraw::element_base *item);
		
		void increaseFollowing(int n);
		void swapItem(KLDrawListItem *i);
		
		void updateVisibleInfo();
		
		const int& id() const { return id_; }
		ldraw::element_base* item() const { return item_; }
		
		void setId(const int &id) { id_ = id; }
		void setItem(ldraw::element_base *item) { item_ = item; }
		
	private:
		void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);
		
		int id_;
		ldraw::element_base *item_;
};

#endif
