// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include <list>

#include <kmainwindow.h>

#include "libldr.h"

class QListViewItem;
class QPopupMenu;
class QTab;
class KAction;
class KRadioAction;
class KRecentFilesAction;
class KURL;
class KWidgetAction;
class KTabBar;
class KTabWidget;
class KLDrawColorPickerAction;
class KLDrawDBManager;
class KLDrawDocument;
class KLDrawEditor;
class KLDrawPartList;
class KLDrawRenderWidget;
class KLDrawList;
class KLDrawListItem;
class KLDrawSubmodelList;
class KLDrawModelProperties;

// Drag & drop resources
struct KLDrawDragResources {
	bool dragging;
	ldraw::matrix dragPos;
	ldraw::vector3d dragBBoxMin;
	ldraw::vector3d dragBBoxMax;
	
	/**
	 * Used only when placing
	 **/
	ldraw::color lastColor;
	ldraw::matrix lastMatrix;
	
	/**
	 * Used only when moving
	 **/
	ldraw::element_extref *dragObject;
	int dragId;
};

class KLDrawMainWindow : public KMainWindow
{
	Q_OBJECT;
	
	static KLDrawMainWindow *instance_;
	
	public:
		KLDrawMainWindow(QWidget *parent = 0, const char *name = 0L);
		~KLDrawMainWindow();
		
		static KLDrawMainWindow* self() { return instance_; }
		
		KLDrawDocument** activeDocument() { return &activeDocument_; }
		KLDrawListItem* getCurrentCursor() { return cursor_; }
		KLDrawEditor* getEditor() { return editor_; }
		KLDrawDragResources* getDragResources() { return &dragResources_; }
		
	signals:
		void editingEnabled(bool);
		void activeModelChanged();
		void documentChanged();
		void refresh();
		
	public slots:
		void newFile();
		void openFile();
		void openRecent(const KURL &url);
		void saveFile();
		void saveFileAs();
		void closeFile();
		
		void changeActiveSelection(int);
		void activate(bool);
		
		void changeCaption();
		
		void resetZoom();
		void resetPersp();
		
		void changeActiveModel(QListViewItem *item);
		
		void enableAction(int id, bool enable);
		
	private slots:
		void changeCursor(KLDrawListItem *item);
		void openModelPropertiesDialog();
		
		void synchronizeViews();
		void dispatchDragStart(const ldraw::vector3d &bboxmin, const ldraw::vector3d &bboxmax);
		void dispatchDragStop();
		
		void submodelNew();
		void submodelModify();
		void submodelDelete();
		
		void render();
		
	private:
		bool initializeLDraw();
		void initializeMenus();
		void setupLayout();
		void setupConnections();
		
		void loadURL(const KURL &location);
		bool saveURL(KLDrawDocument *d, const KURL &location, bool newName);
		void addDocument(KLDrawDocument *newDocument);
		bool queryClose();
		bool checkTab(KLDrawDocument *d);
		bool saveDocument(KLDrawDocument *d);
		bool saveDocumentAs(KLDrawDocument *d);
		
		void sorry();
		
		int blankFiles_;
		
		KLDrawDBManager *db_;
		KLDrawEditor *editor_;
		
		KTabWidget *tabWidget_;
		KLDrawList *listWidget_;
		KLDrawSubmodelList *submodelList_;
		KLDrawPartList *partList_;
		KLDrawRenderWidget *renderWidget_[4];
		KLDrawModelProperties *modelPropertiesWidget_;
		KTabBar *tabBar_;
		std::list<KLDrawDocument *> documentList_;
		KLDrawDocument *activeDocument_;
		KLDrawListItem *cursor_;
		KLDrawDragResources dragResources_;
		
		// File menu
		KAction *newAction;
		KAction *openAction;
		KRecentFilesAction *openRecentAction;
		KAction *saveAction;
		KAction *saveAsAction;
		KAction *closeAction;
		KAction *quitAction;
		
		// Edit menu
		KRadioAction *gridSparseAction;
		KRadioAction *gridStandardAction;
		KRadioAction *gridDenseAction;
		KRadioAction *gridFineAction;
		KAction *rotateXCWAction;
		KAction *rotateXCCWAction;
		KAction *rotateYCWAction;
		KAction *rotateYCCWAction;
		KAction *rotateZCWAction;
		KAction *rotateZCCWAction;
		KAction *deleteAction;
		KAction *moveXMAction;
		KAction *moveXPAction;
		KAction *moveYMAction;
		KAction *moveYPAction;
		KAction *moveZMAction;
		KAction *moveZPAction;
		KLDrawColorPickerAction *colorEditAction;
		KAction *editModelAction;
		
		// Submodel menu
		KAction *submodelNewAction;
		KAction *submodelModifyAction;
		KAction *submodelDeleteAction;
		
		// View menu
		KAction *viewFirstAction;
		KAction *viewPrevAction;
		KAction *viewNextAction;
		KAction *viewLastAction;
		KAction *zoomResetAction;
		KAction *perspResetAction;
		KAction *renderAction;
		KAction *renderOptionsAction;
		
		// Tool menu
};

#endif
