// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <klocale.h>

#include "libldr.h"
#include "mainwindow.h"

#include "modelproperties.h"

using namespace ldraw;

KLDrawModelProperties::KLDrawModelProperties() : QDialog(KLDrawMainWindow::self(), 0L, true)
{
	item_ = 0L;
	nameChanged_ = false;
	
	setCaption(i18n("Model Properties"));
	
	QVBoxLayout *mainLayout = new QVBoxLayout(this, 11, 6);
	
	nameEdit_ = new QLineEdit(this);
	descEdit_ = new QLineEdit(this);
	authorEdit_ = new QLineEdit(this);
	
	QLabel *nameLabel = new QLabel(nameEdit_, i18n("&Name:"), this);
	QLabel *descLabel = new QLabel(descEdit_, i18n("&Description:"), this);
	QLabel *authorLabel = new QLabel(authorEdit_, i18n("&Author:"), this);
	
	mainLayout->addWidget(nameLabel);
	mainLayout->addWidget(nameEdit_);
	mainLayout->addWidget(descLabel);
	mainLayout->addWidget(descEdit_);
	mainLayout->addWidget(authorLabel);
	mainLayout->addWidget(authorEdit_);
	
	mainLayout->addItem(new QSpacerItem(20, 41, QSizePolicy::Minimum, QSizePolicy::Expanding));
	
	QFrame *line = new QFrame(this);
	line->setFrameShape(QFrame::HLine);
	line->setFrameShadow(QFrame::Sunken);
	mainLayout->addWidget(line);
	
	buttonOk_ = new QPushButton(i18n("OK"), this);
	buttonOk_->setDefault(true);
	buttonApply_ = new QPushButton(i18n("Apply"), this);
	buttonCancel_ = new QPushButton(i18n("Cancel"), this);
	buttonCancel_->setAccel(QKeySequence("Esc"));
	
	QHBoxLayout *buttonLayout = new QHBoxLayout(0L, 0, 6);
	buttonLayout->addItem(new QSpacerItem(111, 21, QSizePolicy::Expanding, QSizePolicy::Minimum));
	buttonLayout->addWidget(buttonOk_);
	buttonLayout->addWidget(buttonApply_);
	buttonLayout->addWidget(buttonCancel_);
	mainLayout->addLayout(buttonLayout);
	
	QObject::connect(this, SIGNAL(dataChanged()), KLDrawMainWindow::self(), SLOT(changeCaption()));
	
	QObject::connect(buttonOk_, SIGNAL(clicked()), this, SLOT(actionOK()));
	QObject::connect(buttonApply_, SIGNAL(clicked()), this, SLOT(actionApply()));
	QObject::connect(buttonCancel_, SIGNAL(clicked()), this, SLOT(close()));
	
	refresh();
}

void KLDrawModelProperties::setModel(ldraw::model *m)
{
	item_ = m;
	
	refresh();
}

void KLDrawModelProperties::actionOK()
{
	actionApply();
	close();
	
	emit dataChanged();
}

void KLDrawModelProperties::actionApply()
{
	std::string newName = std::string(nameEdit_->text().local8Bit());
	
	item_->name() = newName;
	item_->desc() = descEdit_->text().local8Bit();
	item_->author() = authorEdit_->text().local8Bit();
	
	if (nameChanged_) { // Update all names in refs according to filename change
		model_multipart *root = item_->parent();
		for (model::iterator it = root->get_model()->begin(); it != root->get_model()->end(); ++it) {
			if ((*it)->get_type() == type_extref && CAST_AS_EXTREF(*it)->get_model() == item_)
				CAST_AS_EXTREF(*it)->get_model()->name() = newName;
		}
		for (std::map<std::string, model *>::iterator it = root->get_submodels().begin(); it != root->get_submodels().end(); ++it) {
			for (model::iterator it2 = (*it).second->begin(); it2 != (*it).second->end(); ++it2) {
				if ((*it2)->get_type() == type_extref && CAST_AS_EXTREF(*it2)->get_model() == item_)
					CAST_AS_EXTREF(*it2)->get_model()->name() = newName;
			}
		}
	}
}

void KLDrawModelProperties::setNameChanged()
{
	nameChanged_ = true;
}

void KLDrawModelProperties::refresh()
{
	if (!item_) {
		setEnabled(false);
		return;
	} else
		setEnabled(true);
	
	model_multipart *root = item_->parent();
	
	nameChanged_ = false;
	
	nameEdit_->setText(QString(item_->name().c_str()));
	descEdit_->setText(QString(item_->desc().c_str()));
	authorEdit_->setText(QString(item_->author().c_str()));
	
	if (root->get_model() == item_) {
		nameEdit_->setEnabled(false);
	} else
		nameEdit_->setEnabled(true);
}

