// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _PARTLIST_H_
#define _PARTLIST_H_

#include <qpixmapcache.h>
#include <qmap.h>
#include <qvbox.h>
#include <klistview.h>
#include <kiconview.h>

#include "libldr.h"

class KToolBarButton;

class KLDrawDBManager;
class KLDrawClickLineEdit;
class KLDrawPartListView;
class KLDrawPartListViewItem;
class KLDrawPartIconView;

class KLDrawPartList : public QVBox
{
	Q_OBJECT;
	
	public:
		KLDrawPartList(QWidget *parent = 0L, const char *name = 0L);
		~KLDrawPartList();
		
	public slots:
		void setEnabled(bool);
		
	private:
		void initializeItems();
		
		KLDrawDBManager *db_;
		
		KToolBarButton *eraseButton_;
		KLDrawClickLineEdit *searchEdit_;
		
		KLDrawPartListView *partList_;
		KLDrawPartIconView *iconList_;
		QPixmapCache thumbnailCache_;
		
		friend class KLDrawPartIconView;
};

class KLDrawPartListView : public KListView
{
	Q_OBJECT;
	
	public:
		KLDrawPartListView(QWidget *parent = 0L, const char *name = 0L);
		~KLDrawPartListView();
		
		KListViewItem* getCategory(const QString &category) { return categoryList_[category]; }
		
		KListViewItem* getRoot() { return partMain_; }
		KListViewItem* getFavorite() { return partFavorite_; }
		
		//virtual void setSorting(int column, bool ascending = true);
		
	public slots:
		void selectItem(KLDrawPartListViewItem *item);
		
	private:
		void registerItem(const QString &category, const QString &desc, const QString &partNo, const QString &filename, const ldraw::vector3d &min, const ldraw::vector3d &max);
		QDragObject* dragObject();
		
		QMap<QString, KListViewItem *> categoryList_;
		KListViewItem *partMain_;
		KListViewItem *partFavorite_;
		
		friend class KLDrawPartList;
};

class KLDrawPartListViewItem : public KListViewItem
{
	public:
		QString desc() { return text(0); }
		QString number() { return text(1); }
		
		const QString& filename() { return filename_; }
		const QString& category() { return category_; }
		const ldraw::vector3d& dimMin() { return min_; }
		const ldraw::vector3d& dimMax() { return max_; }
		
		void setFilename(const QString &fn) { filename_ = fn; }
		void setCategory(const QString &category) { category_ = category; }
		void setDimension(const ldraw::vector3d &min, const ldraw::vector3d &max) { min_ = min, max_ = max; }
	
	private:
		// Not copy-constructible outside KLDrawPartListview
		KLDrawPartListViewItem(KListViewItem *category);
		
		QString filename_;
		QString category_;
		ldraw::vector3d min_;
		ldraw::vector3d max_;
		
		friend class KLDrawPartListView;
};

class KLDrawPartIconView : public KIconView
{
	Q_OBJECT;
	
	public:
		KLDrawPartIconView(KLDrawPartList *top, QWidget *parent = 0L, const char *name = 0L);
		~KLDrawPartIconView();
		
	signals:
		void pointItemInList(KLDrawPartListViewItem *item);
		
	public slots:
		void updateView(QListViewItem *item);
		
	private slots:
		void selectNew(QIconViewItem *item);
		
	private:
		QDragObject* dragObject();
		
		KLDrawPartList *top_;
		QString currentCategory_;
};

class KLDrawPartIconViewItem : public KIconViewItem
{
	public:
		KLDrawPartIconViewItem(KLDrawPartListViewItem *item, KLDrawPartIconView *parent);
		KLDrawPartIconViewItem(KLDrawPartListViewItem *item, KLDrawPartIconView *parent, KLDrawPartIconViewItem *after);
		
		KLDrawPartListViewItem* listItem() { return item_; }
		
	private:
		KLDrawPartListViewItem *item_;
};

#endif
