// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <list>
#include <map>
#include <string>

#include <qtextstream.h>

#include "document.h"
#include "libldr.h"

class KLDrawPOVRayRenderParameters;

class KLDrawPOVRayExporter
{
	public:
		KLDrawPOVRayExporter(KLDrawPOVRayRenderParameters *p) { parameter_ = p; }
		
		void exportModel(QTextStream &stream, KLDrawDocument &doc);
		
	protected:
		KLDrawPOVRayRenderParameters *parameter_;
		
	private:
		QString convertNameFormat(const std::string &s);
		
		void fillHeader(QTextStream &o);
		void fillColors(QTextStream &o, ldraw::model *m);
		void fillColorsRecursive(QTextStream &o, ldraw::model *m, std::list<int> &l);
		void fillParts(QTextStream &o, ldraw::model *m);
		void fillPartsRecursive(QTextStream &o, ldraw::model *m, std::list<ldraw::model *> &l, std::map<QString, char> &bl);
		void fillFooter(QTextStream &o, ldraw::model *m);
		
		// Finishes
		void setPlane(QTextStream &o, ldraw::model *m);
		void setBackground(QTextStream &o);
		void setCamera(QTextStream &o, ldraw::model *m);
		void setLights(QTextStream &o, ldraw::model *m);
		
		// Substitute primitives
		bool substitute(QTextStream &o, QString name);
		void substituteStud(QTextStream &o);
		void substituteStud2(QTextStream &o);
		void substituteStud3(QTextStream &o);
		void substituteStud4(QTextStream &o);
		void substitutePeghole(QTextStream &o);
		void substitute2Dash4Cyli(QTextStream &o);
		void substitute2Dash4Ndis(QTextStream &o);
		void substitute2Dash4Disc(QTextStream &o);
		void substitute4Dash4Cyli(QTextStream &o);
		void substitute4Dash4Ndis(QTextStream &o);
};

