// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qtextstream.h>
#include <ktempfile.h>

#include "povrayrenderparameters.h"

// TODO copy all props

KLDrawPOVRayRenderParameters::KLDrawPOVRayRenderParameters()
{
	width_ = 640;
	height_ = 480;
	antialiasing_ = true;
	
	backgroundColor_ = QColor(0, 0, 0);
	
	drawPlane_ = true;
	planePositionAuto_ = true;
	planePosition_ = 0.0f;
	planeColor_ = QColor(200, 200, 200);
	
	latitude_ = 135.0f;
	longitude_ = 30.0f;
	radius_ = 1.0f;
	fov_ = 67.38f;
	orthographic_ = false;
	
	lights_ = 3;
	lightsLongitude_ = 60.0f;
	lightsLatitudeOffset_ = 45.0f;
	lightsRadius_ = 2.0f;
	lightsIntensity_ = 1.0f;
	lightsColor_ = QColor(255, 255, 255);
}

KLDrawPOVRayRenderParameters::KLDrawPOVRayRenderParameters(const KLDrawPOVRayRenderParameters &p)
{
	width_ = p.width();
	height_ = p.height();
	antialiasing_ = p.antialiasing();
}

KTempFile* KLDrawPOVRayRenderParameters::getIniFile() const
{
	KTempFile *n = new KTempFile(QString::null, ".ini");
	QTextStream &s = *(n->textStream());
	
	s << "Width=" << width_ << "\n";
	s << "Height=" << height_ << "\n";
	s << "Antialias=";
	if (antialiasing_)
		s << "on\nAntialias_Threshold=0.3\n";
	else
		s << "off\n";
	
	n->close();
	n->setAutoDelete(true);
	
	return n;
}

KLDrawPOVRayRenderParameters KLDrawPOVRayRenderParameters::operator= (const KLDrawPOVRayRenderParameters &p) const
{
	KLDrawPOVRayRenderParameters n;
	
	n.setWidth(p.width());
	n.setHeight(p.height());
	n.setAntialiasing(p.antialiasing());
	
	return n;
}
