// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qcstring.h>
#include <qimage.h>
#include <qwidget.h>

#include "povrayexporter.h"

class KTempFile;
class KProcess;
class KLDrawPOVRayRenderParameters;

// Displays POV-Ray rendering progress
// This portion of code has been taken from kpovmodeler.
// KPovModeler (c)2001-2002 Andreas Zehender

class KLDrawPOVRayRenderWidget : public QWidget, public KLDrawPOVRayExporter
{
	Q_OBJECT;
	
	public:
		KLDrawPOVRayRenderWidget(KLDrawPOVRayRenderParameters *param, KLDrawDocument &doc, QWidget *parent, const char *name = 0L);
		~KLDrawPOVRayRenderWidget();
		
		QString povrayOutput() const { return povrayOutput_; }
		QImage image() const { return image_; }
		
		virtual QSize sizeHint() const;
		
	signals:
		void finished(int exitStatus);
		void progress(int percent);
		void lineFinished(int line);
		void povrayMessage(const QString &msg);
		
	public slots:
		void killRendering();
		void suspendRendering();
		void resumeRendering();
		
	private slots:
		void povrayMessage(KProcess *proc, char *buffer, int buflen);
		void povrayImage(KProcess *proc, char *buffer, int buflen);
		void renderingFinished(KProcess *proc);
		
	private:
		void paintEvent(QPaintEvent *e);
		
	private:
		void setPixel(int x, int y, uint c);
		void cleanup();
		
		KProcess *process_;
		bool suspended_;
		QImage image_;
		bool rcvHeader_;
		unsigned char header_[18];
		int rcvHeaderBytes_;
		int skipBytes_;
		int bytespp_;
		int rcvPixels_;
		int progress_;
		unsigned char restBytes_[4];
		int numRestBytes_;
		int line_;
		int column_;
		QString povrayOutput_;
		KTempFile *tempFile_;
		KTempFile *iniFile_;
};
