// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _RENDERER_H_
#define _RENDERER_H_

#include <stack>
#include <GL/gl.h>

#include "libldr.h"

#define ASSUMED_TRANSPARENT_ALPHA 128

class KLDrawRenderer
{
	public:
		enum StudRenderingMode
		{
			studNormal, studLine, studSquare
		};
		
		KLDrawRenderer();
		~KLDrawRenderer();
		
		void initializeColorStack(int id);
		
		void setStudMode(StudRenderingMode s) { studMode = s; }
		void setBoundingBox(bool enable) { boundingBox = enable; }
		
	protected:
		void drawModel(ldraw::model *m, int depth, int cursor = -1);
		void drawBoundingBoxes(ldraw::model *m, int depth, int bypassId = -1);
		
		StudRenderingMode studMode;
		bool shading;
		bool boundingBox;
		
	protected:
		void renderBoundingBox(const ldraw::vector3d &min, const ldraw::vector3d &max, int highlight = 0);
		
	private:
		void getColor(const ldraw::color &c, unsigned char *array);
		void renderLine(ldraw::element_line &l);
		void renderTriangle(ldraw::element_triangle &l);
		void renderQuadrilateral(ldraw::element_quadrilateral &l);
		void renderCondLine(ldraw::element_condline &l, const ldraw::matrix &projectionMatrix);
		void renderStud(ldraw::model *l);
		
		ldraw::vector3d calculateNormal(const ldraw::vector3d &v1, const ldraw::vector3d &v2, const ldraw::vector3d &v3);
		
		std::stack<const ldraw::color_entity *> colorStack;
};

#endif
