// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#ifndef _RENDERWIDGET_H_
#define _RENDERWIDGET_H_

#include <qgl.h>
#include <GL/gl.h>

#include "libldr.h"
#include "renderer.h"

#define MAGNIFICATION_FACTOR 1.375f

class QPoint;
class QMouseEvent;
class QImage;
class KLDrawDocument;
class KLDrawEditor;
class KLDrawMainWindow;
struct KLDrawViewport;
struct KLDrawDragResources;

class KLDrawRenderWidget : public QGLWidget, public KLDrawRenderer
{
	Q_OBJECT;
	
	public:
		enum ViewportMode
		{
			viewportTop, viewportBottom,
			viewportFront, viewportBack,
			viewportLeft, viewportRight,
			viewportFree,
			viewportNull = 255 // Not used
		};
		
		struct KLDrawArcBallRotation
		{
			ldraw::vector3d stVec;
			ldraw::vector3d enVec;
			float adjustedWidth;
			float adjustedHeight;
		};
		
		KLDrawRenderWidget(KLDrawDocument **ptr, ViewportMode viewport, int scrid, QWidget *parent = 0L);
		~KLDrawRenderWidget();
		
		void setControlled(bool enabled) { controlled = enabled; }
		
		ldraw::vector3d viewportCoordinate(const QPoint &dim) const;
		QPoint globalCoordinate(const ldraw::vector3d &vec) const;
		int depthAxis() const;
		
		void dragStart();
		void dragStop();
		
	signals:
		void objectDragging();
		void startDragging(const ldraw::vector3d &, const ldraw::vector3d &);
		void stopDragging();
		
	public slots:
		void initViewport(ViewportMode oldMode, ViewportMode newMode);
		void resize(int width, int height);
		
	protected:
		void setViewport();
		
		void initializeGL();
		void paintGL();
		void resizeGL(int width, int height);
		
		void mousePressEvent(QMouseEvent *event);
		void mouseMoveEvent(QMouseEvent *event);
		void mouseReleaseEvent(QMouseEvent *event);
		void wheelEvent(QWheelEvent *event);
		
	private:
		void renderGrid(float xg, float yg, int xc, int yc, float xo, float yo, float zo);
		void renderViewportName();
		
		ldraw::vector3d mapToSphere(const QPoint &point);
		void rotate();
		
		void dragEnterEvent(QDragEnterEvent *e);
		void dragMoveEvent(QDragMoveEvent *e);
		void dragLeaveEvent(QDragLeaveEvent *e);
		void dropEvent(QDropEvent *e);
		void moveObject(const QPoint &p);
		void contextMenu();
		
		ViewportMode viewportMode;
		int screenId;
		
		bool controlled;
		bool panning;
		bool rotating;
		bool drawGrid;
		
		float windowWidth;
		float windowHeight;
		float windowLength;
		KLDrawArcBallRotation *arcBall;
		
		KLDrawDragResources *dragResources_;
		
		KLDrawDocument **activeDocument_;
		
		// Provided for convenience
		KLDrawViewport *stretched;
		QImage *viewportText;
		
		QPoint lastPos;
		
		KLDrawEditor *editor;
		
		friend class KLDrawMainWindow;
};

#endif
