// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qfontmetrics.h>
#include <qheader.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qstring.h>
#include <klocale.h>

#include "document.h"
#include "mainwindow.h"

#include "submodellist.h"

using namespace std;
using namespace ldraw;

/******************************************************************************
 * KLDrawSubmodelItem
 ******************************************************************************/

KLDrawSubmodelItem::KLDrawSubmodelItem(model *model, QPixmap *pm, QListView *parent)
	: KListViewItem(parent)
{
	isBase_ = false;
	model_ = model;
	setMultiLinesEnabled(true);
	
	setModel(model, pm);
}

KLDrawSubmodelItem::KLDrawSubmodelItem(model *model, QPixmap *pm, QListView *parent, QListViewItem *after)
	: KListViewItem(parent, after)
{
	isBase_ = false;
	model_ = model;
	setMultiLinesEnabled(true);	
	
	setModel(model, pm);
}

int KLDrawSubmodelItem::width(const QFontMetrics &fm, const QListView *lv, int c) const
{
	return QListViewItem::width(fm, lv, c) + 84;
}

void KLDrawSubmodelItem::setModel(ldraw::model *m, QPixmap *p)
{
	model_ = m;
	
	if (isBase_)
		setText(0, i18n("(Base model)"));
	else if (model_)
		setText(0, i18n("%1\n%2").arg(QString::fromLocal8Bit(model_->desc().c_str())).arg(QString::fromLocal8Bit(model_->name().c_str())));
	
	if (p)
		setPixmap(0, *p);
	else
		setPixmap(0, 0);
}

void KLDrawSubmodelItem::setBase(bool enable)
{
	isBase_ = enable;
	
	setText(0, i18n("(Base model)"));
}

/******************************************************************************
 * KLDrawSubmodelList
 ******************************************************************************/

KLDrawSubmodelList::KLDrawSubmodelList(QWidget *parent, const char *name)
	: KListView(parent, name)
{
	addColumn(i18n("Submodels"));
	setColumnAlignment(0, Qt::AlignVCenter);
	setResizeMode(QListView::AllColumns);
	setSorting(-1);
	header()->hide();
	
	baseModel_ = new KLDrawSubmodelItem(0L, 0, this);
	baseModel_->setBase(true);
	
	activeDocument_ = KLDrawMainWindow::self()->activeDocument();
}

KLDrawSubmodelList::~KLDrawSubmodelList()
{
	takeItem(baseModel_);
	delete baseModel_;
}

void KLDrawSubmodelList::updateList()
{
	clearSelection();
	
	// Clear submodels
	for (KLDrawSubmodelItem *iterator = modelList_.first(); iterator; iterator = modelList_.next()) {
		takeItem(iterator);
		delete iterator;
	}
	modelList_.clear();
	
	if (!(*activeDocument_)) {
		baseModel_->setModel(0L);
		baseModel_->setText(0, i18n("(Base model)"));
		return;
	}
	
	baseModel_->setModel((*activeDocument_)->getModel()->get_model(), (*activeDocument_)->getPixmap());
	
	KLDrawSubmodelItem *oldItem = 0L;
	for (map<string, model *>::iterator it = (*activeDocument_)->getModel()->get_submodels().begin(); it != (*activeDocument_)->getModel()->get_submodels().end(); ++it) {
		KLDrawSubmodelItem *newItem = 0L;
		newItem = new KLDrawSubmodelItem((*it).second, (*activeDocument_)->getSubmodelPixmap((*it).first), this, oldItem ? oldItem : baseModel_);
		modelList_.append(newItem);
		oldItem = newItem;
	}
}
