// KLDraw - An interactive LDraw modeler for KDE
// Copyright (c)2006 Park "segfault" J. K. <nemesis@planetmono.org>

#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <klocale.h>

#include "editor.h"
#include "libldr.h"
#include "list.h"
#include "mainwindow.h"

#include "transformdialog.h"

KLDrawTransformDialog::KLDrawTransformDialog()
	: KLDrawCommonDialog()
{
	int i, j;
	
	setCaption(i18n("Edit Matrix"));
	
	QGroupBox *groupVector = new QGroupBox(this);
	QGroupBox *groupMatrix = new QGroupBox(this);
	
	groupVector->setTitle(i18n("Position"));
	groupVector->setColumnLayout(0, Qt::Vertical);
	groupVector->layout()->setSpacing(6);
	groupVector->layout()->setMargin(11);
	groupMatrix->setTitle(i18n("Transformation Matrix"));
	groupMatrix->setColumnLayout(0, Qt::Vertical);
	groupMatrix->layout()->setSpacing(6);
	groupMatrix->layout()->setMargin(11);
	
	// Proper focus order
	for (i = 0; i < 3; ++i) {
		matrixEdit_[3][i] = new QLineEdit(groupVector);
	}
	for (i = 0; i < 3; ++i) {
		for (j = 0; j < 3; ++j) {
			matrixEdit_[i][j] = new QLineEdit(groupMatrix);
		}
	}
	
	QHBoxLayout *groupVectorLayout = new QHBoxLayout(groupVector->layout());
	groupVectorLayout->setAlignment(Qt::AlignTop);
	groupVectorLayout->addWidget(new QLabel(i18n("X :"), groupVector));
	groupVectorLayout->addWidget(matrixEdit_[3][0]);
	groupVectorLayout->addWidget(new QLabel(i18n("Y :"), groupVector));
	groupVectorLayout->addWidget(matrixEdit_[3][1]);
	groupVectorLayout->addWidget(new QLabel(i18n("Z :"), groupVector));
	groupVectorLayout->addWidget(matrixEdit_[3][2]);
	mainLayout_->addWidget(groupVector);
	
	QGridLayout *groupMatrixLayout = new QGridLayout(groupMatrix->layout());
	for (i = 0; i < 3; ++i)
		for (j = 0; j < 3; ++j)
			groupMatrixLayout->addWidget(matrixEdit_[i][j], i, j);
	mainLayout_->addWidget(groupMatrix);
	
	addStandardButtons();
	
	resize(QSize(400, 260).expandedTo(minimumSizeHint()));
	clearWState(WState_Polished);
	
	refresh();
}

KLDrawTransformDialog::~KLDrawTransformDialog()
{
}

void KLDrawTransformDialog::show()
{
	if (!isShown()) {
		matrixEdit_[3][0]->setFocus();
		matrixEdit_[3][0]->selectAll();
	}
	
	QDialog::show();
}

void KLDrawTransformDialog::actionApply()
{
	int i, j;
	
	ldraw::matrix m;
	
	for (i = 0; i < 3; ++i) {
		for (j = 0; j < 3; ++j) {
			m.value(i, j) = matrixEdit_[i][j]->text().toFloat();
		}
		m.value(i, 3) = matrixEdit_[3][i]->text().toFloat();
	}
	
	KLDrawMainWindow::self()->getEditor()->modifyMatrix(item_, m);
}

void KLDrawTransformDialog::refresh()
{
	int i, j;
	
	if (!item_ || item_->item()->get_type() != ldraw::type_extref) {
		setEnabled(false);
	} else {
		setEnabled(true);
		for (i = 0; i < 3; ++i) {
			for (j = 0; j < 3; ++j)
				matrixEdit_[i][j]->setText(QString::number(CAST_AS_EXTREF(item_->item())->get_matrix().value(i, j)));
			matrixEdit_[3][i]->setText(QString::number(CAST_AS_EXTREF(item_->item())->get_matrix().value(i, 3)));
		}
	}
}

